#!/usr/bin/python
'''
    How to run against a clean schroot named 'edgy':
        schroot -c edgy -u root -- sh -c 'apt-get -y install python-unit asp.net2-examples mono-xsp2 && /etc/init.d/mono-xsp2 restart && sleep 10 && ./mono-xsp.py -v'
'''

import unittest, subprocess, os, sys, urllib
#import testlib

class MonoXSPTest(unittest.TestCase):
    '''Test Mono XSP functionality.'''

    def setUp(self):
        '''Setup mechanisms'''

        # Try to figure out (or guess) what port the XSP2 server is on
        self.port = 8082
        try:
                for line in file('/etc/default/mono-xsp2').read().splitlines():
                        if line.find('port=')>=0:
                                self.port = line.split('=',2)[1]
        except IOError:
                # Ignore failures
                return

#    def tearDown(self):

    def _url(self,url):
        return urllib.urlopen('http://localhost:'+self.port+url).read()

    def test_example_okay(self):
        '''Test example server availability'''

        source = self._url('/samples/')
        self.assertTrue(source.find('<title>Welcome to Mono XSP')>=0)

    def test_example_missing(self):
        '''Test bad example server URL'''

        source = self._url('/samples/does-not-exist')
        self.assertTrue(source.find('<title>Error 404</title>')>=0)

    def test_simple_page(self):
        '''Test simple page availability'''

        source = self._url('/samples/2.0/masterpages/simple.aspx')
        self.assertTrue(source.find('Simple Master Page')>=0)

    def test_simple_page_source(self):
        '''Test simple page source not shown (CVE-2006-6104)'''

        source = self._url('/samples/2.0/masterpages/simple.aspx%20')
        self.assertTrue(source.find('<%@ Page Language="C#" MasterPageFile="simple.master" %>')<0)
        self.assertTrue(source.find('<title>Error 404</title>')>=0)

if __name__ == '__main__':
    unittest.main()
