#!/usr/bin/python
'''

    How to run against a clean schroot named 'edgy':
        schroot -c edgy -u root -- sh -c 'apt-get -y install python-unit avahi-daemon avahi-utils python-gdbm && echo "AVAHI_DAEMON_START=1" >> /etc/default/avahi-daemon && /etc/init.d/avahi-daemon restart && ./test-avahi.py -v'
'''


import unittest, os, sys, time, re
import testlib

class AvahiTest(unittest.TestCase):
    '''Test Avahi functionality.'''

    def setUp(self):
        '''Setup Avahi testing'''

        # Figure out which version of Avahi utils are available
        rc, output = testlib.cmd(['/usr/bin/avahi-browse','--help'])
        self.assertEquals(rc,0)

        self.avahi_terminate = False
        if 'Terminate' in output:
            self.avahi_terminate = True


#    def tearDown(self):

    def _browse(self):
        if self.avahi_terminate:
            return testlib.cmd(['/usr/bin/avahi-browse','--all','-t'])
        else:
            return testlib.cmd(['/bin/sh','-c','avahi-browse --all & pid=$!; sleep 4; kill $pid'])

    def _found_service(self,output,service='.*',type='.*',domain='.*'):
        # Figure out which style of report it is
        if re.search('^Service data',output,re.MULTILINE):
            return re.search("^Service data for service '%s' of type .* \(%s\) in domain '%s'.*" % (service,type,domain),output,re.MULTILINE)
        else:
            return re.search('%s\s+%s\s+%s' % (service,type,domain),output,re.MULTILINE)

    def test_avahi_daemon(self):
        '''Test Avahi daemon registrations'''

        rc, output = self._browse()
        self.assertEquals(rc, 0)
        self.assertTrue(self._found_service(output,'.*','Workstation','local'))

    def test_avahi_register_client(self):
        '''Test Avahi client registrations'''

        publisher = os.fork()
        if publisher == 0:
            args = ['/bin/sh','-c','/usr/bin/avahi-publish-service "Unit Test" _http._tcp 12345 >/dev/null 2>&1']
            os.execv(args[0],args)
            sys.exit(0)

        rc, output = self._browse()

        # Kill publisher now, so it always gets cleaned up even on test failure
        os.kill(publisher,15)
        os.waitpid(publisher,0)

        self.assertEquals(rc, 0)
        self.assertTrue(self._found_service(output,'Unit Test','Web Site','local'))

    def test_avahi_register_missing(self):
        '''Test Avahi client mis-registration'''

        rc, output = self._browse()
        self.assertEquals(rc, 0)
        self.assertFalse(self._found_service(output,'Total Nonsense','Blah','DOES NOT EXIST'))

if __name__ == '__main__':
    unittest.main()
