#!/usr/bin/python
'''
    Agh: this file can't be named "dbus.py" because the "import dbus" below
    ends up being recursive?!

    How to run against a clean schroot named 'edgy':
        schroot -c edgy -u root -- sh -c 'apt-get -y install python2.4-unit dbus python2.4-dbus python2.4-gtk2 && ./test-dbus.py -v'
'''


import unittest, subprocess, shutil, grp, os, os.path, sys, time, atexit
import tempfile

import gobject
import dbus
import dbus.service
if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
    import dbus.glib

# Listener goo, to run in parallel to the unitests

class HelloWorldObject(dbus.service.Object):
    def __init__(self, bus_name, object_path='/com/ubuntu/HelloWorldObject'):
        dbus.service.Object.__init__(self, bus_name, object_path)

    @dbus.service.method('com.ubuntu.HelloWorldIFace')
    def hello(self):
        return 'Hello from the HelloWorldObject'

class DBusListener:
    def __init__(self):
        self.listener_pid = os.fork()
        if self.listener_pid == 0:
            try:
                bus_name = dbus.service.BusName('com.ubuntu.HelloWorld', bus=dbus.SessionBus())
                object = HelloWorldObject(bus_name)

                mainloop = gobject.MainLoop()
                mainloop.run()
            except:
                pass
            sys.exit(0)
        time.sleep(1)

    def stop(self):
        if self.listener_pid>0:
            os.kill(self.listener_pid,15)
            os.waitpid(self.listener_pid,0)


class DBusTest(unittest.TestCase):
    '''Test dbus functionality.'''

    def setUp(self):
        self.session_bus = dbus.SessionBus()

    def tearDown(self):
        self.session_bus = None

    def test_dbus_listeners(self):
        '''Test dbus listener registration'''

        proxy_obj = self.session_bus.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
        self.dbus_iface = dbus.Interface(proxy_obj, 'org.freedesktop.DBus')
        listeners = self.dbus_iface.ListNames() 

        self.assertTrue('org.freedesktop.DBus' in listeners)
        self.assertTrue('com.ubuntu.HelloWorld' in listeners)
        self.assertFalse('com.ubuntu.HelloWorldMissing' in listeners)

    def test_dbus_missing(self):
        '''Test dbus exceptions on missing objects'''
        proxy_obj = self.session_bus.get_object( 'com.ubuntu.HelloWorld', '/com/ubuntu/HelloWorldObjectMissing')
        iface = dbus.Interface(proxy_obj, 'com.ubuntu.HelloWorldIFaceMissing')
        self.assertRaises(dbus.DBusException,iface.hello)

    def test_iface_hello(self):
        '''Test dbus object send/receive'''

        proxy_obj = self.session_bus.get_object('com.ubuntu.HelloWorld', '/com/ubuntu/HelloWorldObject')
        iface = dbus.Interface(proxy_obj, 'com.ubuntu.HelloWorldIFace')

        self.assertEquals(iface.hello(),'Hello from the HelloWorldObject')
        self.assertRaises(dbus.DBusException,iface.no_such_function)


if __name__ == '__main__':
    listener = DBusListener()
    atexit.register(listener.stop)

    unittest.main()
