#!/usr/bin/python
'''
    How to run against a clean schroot named 'feisty':
        schroot -c feisty -u root -- sh -c 'apt-get -y install python-unit python-moinmoin apache2  && ./test-moin.py -v'
'''

import unittest, subprocess, urllib
import testlib

class MoinTest(testlib.TestlibCase):
    '''Test moin.'''

    def onetime_setUp(self):
        subprocess.call(['/bin/mkdir', '-p', '/var/www/mywiki','/var/lib/mywiki'], stdout=subprocess.PIPE)
        subprocess.call(['/bin/cp', '-r', '/usr/share/moin/server/moin.cgi','/var/www/mywiki'], stdout=subprocess.PIPE)
        subprocess.call(['/bin/cp', '-r', '/usr/share/moin/data','/usr/share/moin/underlay','/var/lib/mywiki'], stdout=subprocess.PIPE)
        subprocess.call(['/bin/chown', '-R', 'www-data:','/var/www/mywiki','/var/lib/mywiki'], stdout=subprocess.PIPE)

        testlib.config_replace('/etc/apache2/apache2.conf','''
Alias /wiki/ "/usr/share/moin/htdocs/"
ScriptAlias /MyWiki "/var/www/mywiki/moin.cgi"
''', append=True)

        testlib.config_patch('/etc/moin/farmconfig.py','''
diff -urp /etc/moin/farmconfig.py.old /etc/moin/farmconfig.py
--- /etc/moin/farmconfig.py.old	2007-02-15 15:39:05.000000000 -0800
+++ /etc/moin/farmconfig.py	2007-02-15 10:57:24.469369930 -0800
@@ -45,11 +45,12 @@ wikis = [
 
     # wikiname,     url regular expression (no protocol)
     # ---------------------------------------------------------------
-    ("mywiki",  r".*"),   # this is ok for a single wiki
+    #("mywiki",  r".*"),   # this is ok for a single wiki
 
     # for multiple wikis, do something like this:
     #("moinmoin",    r"^moinmoin.wikiwikiweb.de/.*$"),
     #("moinmaster",  r"^moinmaster.wikiwikiweb.de/.*$"),
+    ("mywiki",    r"^localhost/MyWiki/.*$"),
 ]
''')
        testlib.config_patch('/etc/moin/mywiki.py','''
diff -urp /etc/moin/mywiki.py.old /etc/moin/mywiki.py
--- /etc/moin/mywiki.py.old	2007-02-15 15:39:05.000000000 -0800
+++ /etc/moin/mywiki.py	2007-02-15 13:30:22.593314134 -0800
@@ -30,5 +30,7 @@ class Config(FarmConfig):
     # b) if wiki content is maintained in many languages
     page_front_page = u"FrontPage"
 
-    data_dir = '/org/mywiki/data/'
+    data_dir = '/var/lib/mywiki/data'
+    data_underlay_dir = '/var/lib/mywiki/underlay'
 
+    show_traceback = 0
''')
        subprocess.call(['/etc/init.d/apache2', 'stop'], stdout=subprocess.PIPE)
        self.assertTrue(subprocess.call(['/etc/init.d/apache2', 'restart'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT) == 0)

    def onetime_tearDown(self):
        '''Clean up after each test_* function'''
        testlib.config_restore('/etc/moin/mywiki.py')
        testlib.config_restore('/etc/moin/farmconfig.py')
        testlib.config_restore('/etc/apache2/apache2.conf')

    def test_00_basic_page(self):
        '''Default FrontPage available'''
        self.onetime_setUp()
        html = urllib.urlopen('http://localhost/MyWiki/').read()
        self.assertTrue(html.find('FrontPage')>=0, 'Could not find "FrontPage"')

    def test_xss_cve_2007_2423(self):
        '''Safe from AttachFile action error XSS (CVE-2007-2423)'''
        html = urllib.urlopen('http://localhost/MyWiki/FrontPage?action=AttachFile&do=%3Cblink%3EWarning%3C/blink%3E').read()
        self.assertTrue(html.find('Unsupported upload action:')>=0, 'Could not find "Unsupported upload action"')
        self.assertTrue(html.find('<blink>')==-1, 'Found unescaped html tag')
        
    def test_zz_finished(self):
        '''Shutdown'''
        self.onetime_tearDown()
        subprocess.call(['/etc/init.d/apache2', 'stop'], stdout=subprocess.PIPE)
        self.assertTrue(subprocess.call(['/etc/init.d/apache2', 'restart'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT) == 0)

if __name__ == '__main__':
    unittest.main()
